@@ -192,13 +192,6 @@ compute_smooth_frame_time (GdkFrameClock *clock,
    *        and new_frame_time >= old_frame_time. */
   frames_passed = (new_frame_time - smoothed_frame_time_base + frame_interval / 2) / frame_interval;
 
-  if (frames_passed > 1)
-    gdk_profiler_add_markf ((smoothed_frame_time_base - (frame_interval * (frames_passed-1))) * 1000L,
-                            frame_interval * (frames_passed-1) * 1000L,
-                            "Dropped Frames",
-                            "%u frames may have been dropped",
-                            frames_passed-1);
-
   /* We use an approximately whole number of frames in the future from
    * last smoothed frame time. This way we avoid minor jitter in the
    * frame times making the animation speed uneven, but still animate
