@@ -748,6 +748,9 @@ gdk_monitor_set_description (GdkMonitor *monitor,
 double
 gdk_monitor_get_dpi (GdkMonitor *monitor)
 {
-  return MAX ((monitor->geometry.width * monitor->scale) / (monitor->width_mm / MM_PER_INCH),
-              (monitor->geometry.height * monitor->scale) / (monitor->height_mm / MM_PER_INCH));
+  if (monitor->width_mm > 0 && monitor->height_mm > 0)
+    return MAX ((monitor->geometry.width * monitor->scale) / (monitor->width_mm / MM_PER_INCH),
+                (monitor->geometry.height * monitor->scale) / (monitor->height_mm / MM_PER_INCH));
+  else
+    return 96.0;
 }
