@@ -30,6 +30,7 @@
 #include "gdksurface.h"
 
 #include "gdkprivate.h"
+#include "gdkcairoprivate.h"
 #include "gdkcontentprovider.h"
 #include "gdkdeviceprivate.h"
 #include "gdkdisplayprivate.h"
@@ -71,15 +72,10 @@ typedef struct _GdkSurfacePrivate GdkSurfacePrivate;
 
 struct _GdkSurfacePrivate
 {
-  gpointer egl_native_window;
-#ifdef HAVE_EGL
-  EGLSurface egl_surface;
-  GdkMemoryDepth egl_surface_depth;
-#endif
-
   cairo_region_t *opaque_region;
   cairo_rectangle_int_t opaque_rect; /* This is different from the region */
 
+  GdkDrawContext *attached_context;
   gpointer widget;
 
   GdkColorState *color_state;
@@ -507,6 +503,18 @@ gdk_surface_real_get_scale (GdkSurface *surface)
   return 1.0;
 }
 
+static void
+gdk_surface_default_get_buffer_size (GdkSurface      *surface,
+                                     GdkDrawContext  *context,
+                                     guint           *out_width,
+                                     guint           *out_height)
+{
+  double scale = gdk_surface_get_scale (surface);
+
+  *out_width = ceil (scale * surface->width);
+  *out_height = ceil (scale * surface->height);
+}
+
 static GdkSubsurface *
 gdk_surface_real_create_subsurface (GdkSurface *surface)
 {
@@ -543,6 +551,7 @@ gdk_surface_class_init (GdkSurfaceClass *klass)
 
   klass->beep = gdk_surface_real_beep;
   klass->get_scale = gdk_surface_real_get_scale;
+  klass->get_buffer_size = gdk_surface_default_get_buffer_size;
   klass->create_subsurface = gdk_surface_real_create_subsurface;
   klass->set_opaque_region = gdk_surface_default_set_opaque_region;
 
@@ -881,10 +890,10 @@ gdk_surface_get_property (GObject    *object,
 void
 _gdk_surface_update_size (GdkSurface *surface)
 {
-  GSList *l;
+  GdkSurfacePrivate *priv = gdk_surface_get_instance_private (surface);
 
-  for (l = surface->draw_contexts; l; l = l->next)
-    gdk_draw_context_surface_resized (l->data);
+  if (priv->attached_context)
+    gdk_draw_context_surface_resized (priv->attached_context);
 
   g_object_notify (G_OBJECT (surface), "width");
   g_object_notify (G_OBJECT (surface), "height");
@@ -991,10 +1000,13 @@ _gdk_surface_destroy_hierarchy (GdkSurface *surface,
   if (GDK_SURFACE_DESTROYED (surface))
     return;
 
-  GDK_SURFACE_GET_CLASS (surface)->destroy (surface, foreign_destroy);
+  if (priv->attached_context)
+    {
+      gdk_draw_context_detach (priv->attached_context);
+      g_assert (priv->attached_context == NULL);
+    }
 
-  /* backend must have unset this */
-  g_assert (priv->egl_native_window == NULL);
+  GDK_SURFACE_GET_CLASS (surface)->destroy (surface, foreign_destroy);
 
   if (surface->gl_paint_context)
     {
@@ -1130,111 +1142,6 @@ gdk_surface_get_mapped (GdkSurface *surface)
   return GDK_SURFACE_IS_MAPPED (surface);
 }
 
-void
-gdk_surface_set_egl_native_window (GdkSurface *self,
-                                   gpointer    native_window)
-{
-#ifdef HAVE_EGL
-  GdkSurfacePrivate *priv = gdk_surface_get_instance_private (self);
-  GdkGLContext *current = NULL;
-
-  /* This checks that all EGL platforms we support conform to the same struct sizes.
-   * When this ever fails, there will be some fun times happening for whoever tries
-   * this weird EGL backend... */
-  G_STATIC_ASSERT (sizeof (gpointer) == sizeof (EGLNativeWindowType));
-
-  if (priv->egl_surface != NULL)
-    {
-      GdkDisplay *display = gdk_surface_get_display (self);
-
-      current = gdk_gl_context_clear_current_if_surface (self);
-
-      eglDestroySurface (gdk_display_get_egl_display (display), priv->egl_surface);
-      priv->egl_surface = NULL;
-    }
-
-  priv->egl_native_window = native_window;
-
-  if (current)
-    {
-      gdk_gl_context_make_current (current);
-      g_object_unref (current);
-    }
-}
-
-gpointer /* EGLSurface */
-gdk_surface_get_egl_surface (GdkSurface *self)
-{
-  GdkSurfacePrivate *priv = gdk_surface_get_instance_private (self);
-
-  return priv->egl_surface;
-}
-
-GdkMemoryDepth
-gdk_surface_ensure_egl_surface (GdkSurface     *self,
-                                GdkMemoryDepth  depth)
-{
-  GdkSurfacePrivate *priv = gdk_surface_get_instance_private (self);
-  GdkDisplay *display = gdk_surface_get_display (self);
-
-  g_return_val_if_fail (priv->egl_native_window != NULL, depth);
-
-  if (depth == GDK_MEMORY_NONE)
-    {
-      if (priv->egl_surface_depth == GDK_MEMORY_NONE)
-        depth = GDK_MEMORY_U8;
-      else
-        depth = priv->egl_surface_depth;
-    }
-
-  if (priv->egl_surface == NULL ||
-      (priv->egl_surface != NULL &&
-       gdk_display_get_egl_config (display, priv->egl_surface_depth) != gdk_display_get_egl_config (display, depth)))
-    {
-      GdkGLContext *cleared;
-      EGLint attribs[4];
-      int i;
-
-      cleared = gdk_gl_context_clear_current_if_surface (self);
-      if (priv->egl_surface != NULL)
-        eglDestroySurface (gdk_display_get_egl_display (display), priv->egl_surface);
-
-      i = 0;
-      if (depth == GDK_MEMORY_U8_SRGB && display->have_egl_gl_colorspace)
-        {
-          attribs[i++] = EGL_GL_COLORSPACE_KHR;
-          attribs[i++] = EGL_GL_COLORSPACE_SRGB_KHR;
-          self->is_srgb = TRUE;
-        }
-      g_assert (i < G_N_ELEMENTS (attribs));
-      attribs[i++] = EGL_NONE;
-
-      priv->egl_surface = eglCreateWindowSurface (gdk_display_get_egl_display (display),
-                                                  gdk_display_get_egl_config (display, depth),
-                                                  (EGLNativeWindowType) priv->egl_native_window,
-                                                  attribs);
-      if (priv->egl_surface == EGL_NO_SURFACE)
-        {
-          /* just assume the error is no srgb support and try again without */
-          self->is_srgb = FALSE;
-          priv->egl_surface = eglCreateWindowSurface (gdk_display_get_egl_display (display),
-                                                      gdk_display_get_egl_config (display, depth),
-                                                      (EGLNativeWindowType) priv->egl_native_window,
-                                                      NULL);
-        }
-      priv->egl_surface_depth = depth;
-
-      if (cleared)
-        {
-          gdk_gl_context_make_current (cleared);
-          g_object_unref (cleared);
-        }
-    }
-
-  return priv->egl_surface_depth;
-#endif
-}
-
 gboolean
 gdk_surface_get_gl_is_srgb (GdkSurface *self)
 {
@@ -1298,6 +1205,9 @@ gdk_surface_create_gl_context (GdkSurface   *surface,
  * Creates a new `GdkCairoContext` for rendering on @surface.
  *
  * Returns: (transfer full): the newly created `GdkCairoContext`
+ *
+ * Deprecated: 4.18: Drawing content with Cairo should be done via
+ *   Cairo rendernodes, not by using the Cairo renderer.
  */
 GdkCairoContext *
 gdk_surface_create_cairo_context (GdkSurface *surface)
@@ -1355,7 +1265,8 @@ gdk_surface_schedule_update (GdkSurface *surface)
 
   /* If there's no frame clock (a foreign surface), then the invalid
    * region will just stick around unless gdk_surface_process_updates()
-   * is called. */
+   * is called.
+   */
   frame_clock = gdk_surface_get_frame_clock (surface);
   if (frame_clock)
     gdk_frame_clock_request_phase (gdk_surface_get_frame_clock (surface),
@@ -2398,8 +2309,7 @@ gdk_surface_create_similar_surface (GdkSurface      *surface,
 
   scale = gdk_surface_get_scale_factor (surface);
 
-  similar_surface = cairo_image_surface_create (content == CAIRO_CONTENT_COLOR ? CAIRO_FORMAT_RGB24 :
-                                                content == CAIRO_CONTENT_ALPHA ? CAIRO_FORMAT_A8 : CAIRO_FORMAT_ARGB32,
+  similar_surface = cairo_image_surface_create (gdk_cairo_format_for_content (content),
                                                 width * scale, height * scale);
   cairo_surface_set_device_scale (similar_surface, scale, scale);
 
@@ -2829,11 +2739,17 @@ gdk_surface_queue_state_change (GdkSurface       *surface,
                                 GdkToplevelState  unset_flags,
                                 GdkToplevelState  set_flags)
 {
+  GdkFrameClock *frame_clock;
+
   surface->pending_unset_flags |= unset_flags;
   surface->pending_set_flags &= ~unset_flags;
 
   surface->pending_set_flags |= set_flags;
   surface->pending_unset_flags &= ~set_flags;
+
+  frame_clock = gdk_surface_get_frame_clock (surface);
+  if (!frame_clock || gdk_frame_clock_is_frozen (frame_clock))
+    gdk_surface_apply_state_change (surface);
 }
 
 void
@@ -3050,6 +2966,7 @@ add_event_mark (GdkEvent *event,
     case GDK_PAD_BUTTON_RELEASE:
     case GDK_PAD_RING:
     case GDK_PAD_STRIP:
+    case GDK_PAD_DIAL:
     case GDK_PAD_GROUP_MODE:
     case GDK_GRAB_BROKEN:
     case GDK_DELETE:
@@ -3251,3 +3168,30 @@ gdk_surface_set_color_state (GdkSurface    *surface,
 
   gdk_surface_invalidate_rect (surface, NULL);
 }
+
+/*<private>
+ * gdk_surface_set_attached_context:
+ * @self: the surface
+ * @context: (nullable): the context to attach
+ *
+ * This function is an implementation detail for GdkDrawContext.
+ *
+ * Use draw context functions like gdk_draw_context_attach() or
+ * gdk_draw_context_detach().
+ **/
+void
+gdk_surface_set_attached_context (GdkSurface     *self,
+                                  GdkDrawContext *context)
+{
+  GdkSurfacePrivate *priv = gdk_surface_get_instance_private (self);
+  
+  priv->attached_context = context;
+}
+
+GdkDrawContext *
+gdk_surface_get_attached_context (GdkSurface *self)
+{
+  GdkSurfacePrivate *priv = gdk_surface_get_instance_private (self);
+
+  return priv->attached_context;
+}
