@@ -17,8 +17,7 @@
  * SPDX-License-Identifier: LGPL-2.1-or-later
  */
 
-#ifndef __GDK_MACOS_EVENT_SOURCE_PRIVATE_H__
-#define __GDK_MACOS_EVENT_SOURCE_PRIVATE_H__
+#pragma once
 
 #include <AppKit/AppKit.h>
 
@@ -37,4 +36,3 @@ gboolean  _gdk_macos_event_source_check_pending (void);
 
 G_END_DECLS
 
-#endif /* __GDK_MACOS_EVENT_SOURCE_PRIVATE_H__ */
