@@ -24,6 +24,8 @@
 #include <OpenGL/CGLIOSurface.h>
 #include <QuartzCore/QuartzCore.h>
 
+#import "GdkMacosLayer.h"
+
 #include "gdkmacosbuffer-private.h"
 #include "gdkmacosglcontext-private.h"
 #include "gdkmacossurface-private.h"
@@ -479,6 +481,7 @@ gdk_macos_gl_context_real_realize (GdkGLContext  *context,
 
 static void
 gdk_macos_gl_context_begin_frame (GdkDrawContext  *context,
+                                  gpointer         context_data,
                                   GdkMemoryDepth   depth,
                                   cairo_region_t  *region,
                                   GdkColorState  **out_color_state,
@@ -500,7 +503,7 @@ gdk_macos_gl_context_begin_frame (GdkDrawContext  *context,
   gdk_gl_context_make_current (GDK_GL_CONTEXT (self));
   gdk_macos_gl_context_allocate (self);
 
-  GDK_DRAW_CONTEXT_CLASS (gdk_macos_gl_context_parent_class)->begin_frame (context, depth, region, out_color_state, out_depth);
+  GDK_DRAW_CONTEXT_CLASS (gdk_macos_gl_context_parent_class)->begin_frame (context, context_data, depth, region, out_color_state, out_depth);
 
   gdk_gl_context_make_current (GDK_GL_CONTEXT (self));
   CHECK_GL (NULL, glBindFramebuffer (GL_FRAMEBUFFER, self->fbo));
@@ -508,6 +511,7 @@ gdk_macos_gl_context_begin_frame (GdkDrawContext  *context,
 
 static void
 gdk_macos_gl_context_end_frame (GdkDrawContext *context,
+                                gpointer        context_data,
                                 cairo_region_t *painted)
 {
   GdkMacosGLContext *self = GDK_MACOS_GL_CONTEXT (context);
@@ -518,7 +522,7 @@ gdk_macos_gl_context_end_frame (GdkDrawContext *context,
   g_assert (GDK_IS_MACOS_GL_CONTEXT (self));
   g_assert (self->cgl_context != nil);
 
-  GDK_DRAW_CONTEXT_CLASS (gdk_macos_gl_context_parent_class)->end_frame (context, painted);
+  GDK_DRAW_CONTEXT_CLASS (gdk_macos_gl_context_parent_class)->end_frame (context, context_data, painted);
 
   surface = gdk_draw_context_get_surface (context);
   gdk_gl_context_make_current (GDK_GL_CONTEXT (self));
@@ -563,6 +567,24 @@ gdk_macos_gl_context_surface_resized (GdkDrawContext *draw_context)
     CGLUpdateContext (self->cgl_context);
 }
 
+
+static gboolean
+gdk_macos_gl_context_surface_attach (GdkDrawContext  *context,
+                                     GError         **error)
+{
+  GdkSurface *surface = gdk_draw_context_get_surface (context);
+  NSView *view = _gdk_macos_surface_get_view (GDK_MACOS_SURFACE (surface));
+
+  if (!GDK_IS_MACOS_LAYER ([view layer]))
+    {
+      [view setLayerContentsRedrawPolicy:NSViewLayerContentsRedrawNever];
+      [view setLayer:[GdkMacosLayer layer]];
+      [view setWantsLayer:YES];
+    }
+
+  return TRUE;
+}
+
 static gboolean
 gdk_macos_gl_context_clear_current (GdkGLContext *context)
 {
@@ -676,6 +698,7 @@ gdk_macos_gl_context_class_init (GdkMacosGLContextClass *klass)
   draw_context_class->end_frame = gdk_macos_gl_context_end_frame;
   draw_context_class->empty_frame = gdk_macos_gl_context_empty_frame;
   draw_context_class->surface_resized = gdk_macos_gl_context_surface_resized;
+  draw_context_class->surface_attach = gdk_macos_gl_context_surface_attach;
 
   gl_class->get_damage = gdk_macos_gl_context_get_damage;
   gl_class->clear_current = gdk_macos_gl_context_clear_current;
