@@ -19,30 +19,14 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __GDK_WAYLAND_GL_CONTEXT__
-#define __GDK_WAYLAND_GL_CONTEXT__
+#pragma once
 
 #include "gdkglcontextprivate.h"
-#include "gdkdisplayprivate.h"
-#include "gdksurface.h"
-
-#include <epoxy/egl.h>
 
 G_BEGIN_DECLS
 
-struct _GdkWaylandGLContext
-{
-  GdkGLContext parent_instance;
-};
-
-struct _GdkWaylandGLContextClass
-{
-  GdkGLContextClass parent_class;
-};
-
 GdkGLContext *  gdk_wayland_display_init_gl                         (GdkDisplay        *display,
                                                                      GError           **error);
 
 G_END_DECLS
 
-#endif /* __GDK_WAYLAND_GL_CONTEXT__ */
