@@ -26,11 +26,12 @@
 
 #define __GDKWIN32_H_INSIDE__
 
+#include <gdk/win32/gdkd3d12texture.h>
+#include <gdk/win32/gdkd3d12texturebuilder.h>
 #include <gdk/win32/gdkwin32cursor.h>
 #include <gdk/win32/gdkwin32display.h>
 #include <gdk/win32/gdkwin32displaymanager.h>
 #include <gdk/win32/gdkwin32dnd.h>
-#include <gdk/win32/gdkwin32screen.h>
 #include <gdk/win32/gdkwin32surface.h>
 #include <gdk/win32/gdkwin32misc.h>
 #include <gdk/win32/gdkwin32monitor.h>
