@@ -1526,17 +1526,6 @@ gdk_x11_device_manager_xi2_translate_event (GdkEventTranslator *translator,
         GdkTranslatedKey translated;
         GdkTranslatedKey no_lock;
 
-        GDK_DISPLAY_DEBUG (display, EVENTS,
-                           "key %s:\twindow %ld\n"
-                           "\tdevice:%u\n"
-                           "\tsource device:%u\n"
-                           "\tkey number: %u\n",
-                           (ev->evtype == XI_KeyPress) ? "press" : "release",
-                           xev->event,
-                           xev->deviceid,
-                           xev->sourceid,
-                           xev->detail);
-
         state = _gdk_x11_device_xi2_translate_state (&xev->mods, &xev->buttons, &xev->group);
 
         device = g_hash_table_lookup (device_manager->id_table,
@@ -1558,6 +1547,18 @@ gdk_x11_device_manager_xi2_translate_event (GdkEventTranslator *translator,
         _gdk_x11_keymap_add_virt_mods (keymap, &state);
         state |= orig_state;
 
+        GDK_DISPLAY_DEBUG (display, EVENTS,
+                           "key %s:\twindow %ld\n"
+                           "\tdevice:%u\n"
+                           "\tsource device:%u\n"
+                           "\tkey number: %u (%s)\n",
+                           (ev->evtype == XI_KeyPress) ? "press" : "release",
+                           xev->event,
+                           xev->deviceid,
+                           xev->sourceid,
+                           xev->detail,
+                           gdk_keyval_name (keyval));
+
         translated.keyval = keyval;
         translated.consumed = consumed;
         translated.layout = layout;
@@ -1658,7 +1659,8 @@ gdk_x11_device_manager_xi2_translate_event (GdkEventTranslator *translator,
                                                    NULL,
                                                    xev->time,
                                                    _gdk_x11_device_xi2_translate_state (&xev->mods, &xev->buttons, &xev->group),
-                                                   direction);
+                                                   direction,
+                                                   GDK_SCROLL_RELATIVE_DIRECTION_UNKNOWN);
 
           }
         else
@@ -1759,7 +1761,8 @@ gdk_x11_device_manager_xi2_translate_event (GdkEventTranslator *translator,
                                                        NULL,
                                                        xev->time,
                                                        state,
-                                                       direction);
+                                                       direction,
+                                                       GDK_SCROLL_RELATIVE_DIRECTION_UNKNOWN);
               }
             else if (gdk_device_get_source (source_device) == GDK_SOURCE_MOUSE)
               {
@@ -1770,7 +1773,8 @@ gdk_x11_device_manager_xi2_translate_event (GdkEventTranslator *translator,
                                                        state,
                                                        direction,
                                                        delta_x * 120.0,
-                                                       delta_y * 120.0);
+                                                       delta_y * 120.0,
+                                                       GDK_SCROLL_RELATIVE_DIRECTION_UNKNOWN);
               }
             else
               {
@@ -1782,7 +1786,8 @@ gdk_x11_device_manager_xi2_translate_event (GdkEventTranslator *translator,
                                               delta_x,
                                               delta_y,
                                               delta_x == 0.0 && delta_y == 0.0,
-                                              GDK_SCROLL_UNIT_WHEEL);
+                                              GDK_SCROLL_UNIT_WHEEL,
+                                              GDK_SCROLL_RELATIVE_DIRECTION_UNKNOWN);
               }
             break;
           }
