@@ -8,8 +8,16 @@
 #include "gsktransformprivate.h"
 #include "gskrendererprivate.h"
 #include "gskrendernodeprivate.h"
+#include "gdk/gdkcolorstateprivate.h"
 #include "gdk/gdktextureprivate.h"
 
+/**
+ * GskBroadwayRenderer:
+ *
+ * A Broadway based renderer.
+ *
+ * See [class@Gsk.Renderer].
+ */
 struct _GskBroadwayRenderer
 {
   GskRenderer parent_instance;
@@ -37,6 +45,7 @@ static gboolean
 gsk_broadway_renderer_realize (GskRenderer  *renderer,
                                GdkDisplay   *display,
                                GdkSurface   *surface,
+                               gboolean      attach,
                                GError      **error)
 {
   GskBroadwayRenderer *self = GSK_BROADWAY_RENDERER (renderer);
@@ -49,6 +58,11 @@ gsk_broadway_renderer_realize (GskRenderer  *renderer,
     }
 
   self->draw_context = gdk_broadway_draw_context_context (surface);
+  if (attach && !gdk_draw_context_attach (GDK_DRAW_CONTEXT (self->draw_context), error))
+    {
+      g_clear_object (&self->draw_context);
+      return FALSE;
+    }
 
   return TRUE;
 }
@@ -57,6 +71,9 @@ static void
 gsk_broadway_renderer_unrealize (GskRenderer *renderer)
 {
   GskBroadwayRenderer *self = GSK_BROADWAY_RENDERER (renderer);
+
+  gdk_draw_context_detach (GDK_DRAW_CONTEXT (self->draw_context));
+
   g_clear_object (&self->draw_context);
 }
 
@@ -275,6 +292,7 @@ collect_reused_child_nodes (GskRenderer *renderer,
     case GSK_FILL_NODE:
     case GSK_STROKE_NODE:
     case GSK_SUBSURFACE_NODE:
+    case GSK_COMPONENT_TRANSFER_NODE:
 
     default:
 
@@ -477,7 +495,7 @@ get_colorized_texture (GdkTexture *texture,
         return g_object_ref (colorized->texture);
     }
 
-  surface = gdk_texture_download_surface (texture);
+  surface = gdk_texture_download_surface (texture, GDK_COLOR_STATE_SRGB);
   image_surface = cairo_surface_map_to_image (surface, NULL);
   data = cairo_image_surface_get_data (image_surface);
   width = cairo_image_surface_get_width (image_surface);
@@ -873,6 +891,7 @@ gsk_broadway_renderer_add_node (GskRenderer *renderer,
     case GSK_GL_SHADER_NODE:
     case GSK_FILL_NODE:
     case GSK_STROKE_NODE:
+    case GSK_COMPONENT_TRANSFER_NODE:
     default:
       break; /* Fallback */
     }
@@ -926,7 +945,7 @@ gsk_broadway_renderer_render (GskRenderer          *renderer,
 
   self->node_lookup = g_hash_table_new (g_direct_hash, g_direct_equal);
 
-  gdk_draw_context_begin_frame (GDK_DRAW_CONTEXT (self->draw_context), update_area);
+  gdk_draw_context_begin_frame_full (GDK_DRAW_CONTEXT (self->draw_context), NULL, GDK_MEMORY_U8, update_area, NULL);
 
   /* These are owned by the draw context between begin and end, but
      cache them here for easier access during the render */
@@ -938,7 +957,7 @@ gsk_broadway_renderer_render (GskRenderer          *renderer,
   self->nodes = NULL;
   self->node_textures = NULL;
 
-  gdk_draw_context_end_frame (GDK_DRAW_CONTEXT (self->draw_context));
+  gdk_draw_context_end_frame_full (GDK_DRAW_CONTEXT (self->draw_context), NULL);
 
   if (self->last_node_lookup)
     g_hash_table_unref (self->last_node_lookup);
@@ -989,6 +1008,8 @@ gsk_broadway_renderer_init (GskBroadwayRenderer *self)
  * support.
  *
  * Returns: a new Broadway renderer.
+ *
+ * Deprecated: 4.20: Broadway will be retired in GTK 5
  **/
 GskRenderer *
 gsk_broadway_renderer_new (void)
