@@ -17,8 +17,7 @@ struct _GskGpuConvertCicpOp
 };
 
 #define VARIATION_OPACITY              (1u << 0)
-#define VARIATION_STRAIGHT_ALPHA       (1u << 1)
-#define VARIATION_REVERSE              (1u << 2)
+#define VARIATION_REVERSE              (1u << 1)
 
 static void
 gsk_gpu_convert_cicp_op_print_instance (GskGpuShaderOp *shader,
@@ -29,14 +28,13 @@ gsk_gpu_convert_cicp_op_print_instance (GskGpuShaderOp *shader,
 
   gsk_gpu_print_rect (string, instance->rect);
   gsk_gpu_print_image (string, shader->images[0]);
-  if (shader->variation & VARIATION_STRAIGHT_ALPHA)
-    gsk_gpu_print_string (string, "straight");
   if (shader->variation & VARIATION_REVERSE)
     gsk_gpu_print_string (string, "reverse");
   g_string_append_printf (string, "cicp %u/%u/%u/%u",
                           instance->color_primaries,
                           instance->transfer_function,
-                          0, 1);
+                          instance->matrix_coefficients,
+                          instance->range);
 }
 
 static const GskGpuShaderOpClass GSK_GPU_CONVERT_CICP_OP_CLASS = {
@@ -67,7 +65,6 @@ gsk_gpu_convert_from_cicp_op (GskGpuFrame             *frame,
                               const GdkCicp           *cicp,
                               GskGpuColorStates        color_states,
                               float                    opacity,
-                              gboolean                 straight_alpha,
                               const graphene_point_t  *offset,
                               const GskGpuShaderImage *image)
 {
@@ -76,8 +73,7 @@ gsk_gpu_convert_from_cicp_op (GskGpuFrame             *frame,
   gsk_gpu_shader_op_alloc (frame,
                            &GSK_GPU_CONVERT_CICP_OP_CLASS,
                            color_states,
-                           (opacity < 1.0 ? VARIATION_OPACITY : 0) |
-                             (straight_alpha ? VARIATION_STRAIGHT_ALPHA : 0),
+                           (opacity < 1.0 ? VARIATION_OPACITY : 0),
                            clip,
                            (GskGpuImage *[1]) { image->image },
                            (GskGpuSampler[1]) { image->sampler },
@@ -88,6 +84,8 @@ gsk_gpu_convert_from_cicp_op (GskGpuFrame             *frame,
   instance->opacity = opacity;
   instance->color_primaries = cicp->color_primaries;
   instance->transfer_function = cicp->transfer_function;
+  instance->matrix_coefficients = cicp->matrix_coefficients;
+  instance->range = cicp->range == GDK_CICP_RANGE_NARROW ? 0 : 1;
 }
 
 void
@@ -96,7 +94,6 @@ gsk_gpu_convert_to_cicp_op (GskGpuFrame             *frame,
                             const GdkCicp           *cicp,
                             GskGpuColorStates        color_states,
                             float                    opacity,
-                            gboolean                 straight_alpha,
                             const graphene_point_t  *offset,
                             const GskGpuShaderImage *image)
 {
@@ -106,8 +103,7 @@ gsk_gpu_convert_to_cicp_op (GskGpuFrame             *frame,
                            &GSK_GPU_CONVERT_CICP_OP_CLASS,
                            color_states,
                            (opacity < 1.0 ? VARIATION_OPACITY : 0) |
-                             (straight_alpha ? VARIATION_STRAIGHT_ALPHA : 0) |
-                             VARIATION_REVERSE,
+                           VARIATION_REVERSE,
                            clip,
                            (GskGpuImage *[1]) { image->image },
                            (GskGpuSampler[1]) { image->sampler },
@@ -118,4 +114,6 @@ gsk_gpu_convert_to_cicp_op (GskGpuFrame             *frame,
   instance->opacity = opacity;
   instance->color_primaries = cicp->color_primaries;
   instance->transfer_function = cicp->transfer_function;
+  instance->matrix_coefficients = cicp->matrix_coefficients;
+  instance->range = cicp->range == GDK_CICP_RANGE_NARROW ? 0 : 1;
 }
