@@ -2,6 +2,8 @@
 
 #include <gsk/gskglshader.h>
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
+
 G_BEGIN_DECLS
 
 typedef struct {
@@ -15,3 +17,5 @@ const GskGLUniform *gsk_gl_shader_get_uniforms (GskGLShader *shader,
 
 G_END_DECLS
 
+G_GNUC_END_IGNORE_DEPRECATIONS
+
