@@ -36,6 +36,9 @@ struct _GtkCssTokenizer
   const char            *end;
 
   GtkCssLocation         position;
+
+  GtkCssLocation         saved_position;
+  const char            *saved_data;
 };
 
 void
@@ -568,6 +571,14 @@ gtk_css_token_init_dimension (GtkCssToken     *token,
 
 GtkCssTokenizer *
 gtk_css_tokenizer_new (GBytes *bytes)
+{
+  return gtk_css_tokenizer_new_for_range (bytes, 0, g_bytes_get_size (bytes));
+}
+
+GtkCssTokenizer *
+gtk_css_tokenizer_new_for_range (GBytes *bytes,
+                                 gsize   offset,
+                                 gsize   length)
 {
   GtkCssTokenizer *tokenizer;
 
@@ -576,8 +587,8 @@ gtk_css_tokenizer_new (GBytes *bytes)
   tokenizer->bytes = g_bytes_ref (bytes);
   tokenizer->name_buffer = g_string_new (NULL);
 
-  tokenizer->data = g_bytes_get_data (bytes, NULL);
-  tokenizer->end = tokenizer->data + g_bytes_get_size (bytes);
+  tokenizer->data = g_bytes_get_region (bytes, 1, offset, length);
+  tokenizer->end = tokenizer->data + length;
 
   gtk_css_location_init (&tokenizer->position);
 
@@ -604,6 +615,12 @@ gtk_css_tokenizer_unref (GtkCssTokenizer *tokenizer)
   g_free (tokenizer);
 }
 
+GBytes *
+gtk_css_tokenizer_get_bytes (GtkCssTokenizer *tokenizer)
+{
+  return tokenizer->bytes;
+}
+
 const GtkCssLocation *
 gtk_css_tokenizer_get_location (GtkCssTokenizer *tokenizer)
 {
@@ -1314,7 +1331,8 @@ gtk_css_tokenizer_read_token (GtkCssTokenizer  *tokenizer,
 
     case '#':
       gtk_css_tokenizer_consume_ascii (tokenizer);
-      if (is_name (*tokenizer->data) || gtk_css_tokenizer_has_valid_escape (tokenizer))
+      if (gtk_css_tokenizer_remaining (tokenizer) > 0 &&
+          (is_name (*tokenizer->data) || gtk_css_tokenizer_has_valid_escape (tokenizer)))
         {
           GtkCssTokenType type;
 
@@ -1484,3 +1502,23 @@ gtk_css_tokenizer_read_token (GtkCssTokenizer  *tokenizer,
     }
 }
 
+void
+gtk_css_tokenizer_save (GtkCssTokenizer *tokenizer)
+{
+  g_assert (!tokenizer->saved_data);
+
+  tokenizer->saved_position = tokenizer->position;
+  tokenizer->saved_data = tokenizer->data;
+}
+
+void
+gtk_css_tokenizer_restore (GtkCssTokenizer *tokenizer)
+{
+  g_assert (tokenizer->saved_data);
+
+  tokenizer->position = tokenizer->saved_position;
+  tokenizer->data = tokenizer->saved_data;
+
+  gtk_css_location_init (&tokenizer->saved_position);
+  tokenizer->saved_data = NULL;
+}
