@@ -15,8 +15,7 @@
  * License along with this library. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#ifndef __GTK_ADJUSTMENT_PRIVATE_H__
-#define __GTK_ADJUSTMENT_PRIVATE_H__
+#pragma once
 
 
 #include <gtk/gtkadjustment.h>
@@ -24,6 +23,8 @@
 
 G_BEGIN_DECLS
 
+double gtk_adjustment_get_bounded_upper (GtkAdjustment *self);
+
 void gtk_adjustment_enable_animation (GtkAdjustment *adjustment,
                                       GdkFrameClock *clock,
                                       guint          duration);
@@ -37,4 +38,3 @@ gboolean gtk_adjustment_is_animating (GtkAdjustment *adjustment);
 G_END_DECLS
 
 
-#endif /* __GTK_ADJUSTMENT_PRIVATE_H__ */
