@@ -20,13 +20,12 @@
 #include "gtkcolorswatchprivate.h"
 
 #include "gtkbox.h"
-#include "gtkcolorchooserprivate.h"
+#include "deprecated/gtkcolorchooserprivate.h"
 #include "gtkdragsource.h"
 #include "gtkdroptarget.h"
 #include "gtkgesturelongpress.h"
 #include "gtkgestureclick.h"
 #include "gtkgesturesingle.h"
-#include "gtkicontheme.h"
 #include "gtkimage.h"
 #include <glib/gi18n-lib.h>
 #include "gtkmain.h"
