@@ -53,13 +53,11 @@ gtk_css_value_font_features_free (GtkCssValue *value)
 }
 
 static GtkCssValue *
-gtk_css_value_font_features_compute (GtkCssValue      *specified,
-                                     guint             property_id,
-                                     GtkStyleProvider *provider,
-                                     GtkCssStyle      *style,
-                                     GtkCssStyle      *parent_style)
+gtk_css_value_font_features_compute (GtkCssValue          *specified,
+                                     guint                 property_id,
+                                     GtkCssComputeContext *context)
 {
-  return _gtk_css_value_ref (specified);
+  return gtk_css_value_ref (specified);
 }
 
 static gboolean
@@ -158,6 +156,7 @@ static const GtkCssValueClass GTK_CSS_VALUE_FONT_FEATURES = {
   "GtkCssFontFeaturesValue",
   gtk_css_value_font_features_free,
   gtk_css_value_font_features_compute,
+  NULL,
   gtk_css_value_font_features_equal,
   gtk_css_value_font_features_transition,
   NULL,
@@ -170,7 +169,7 @@ gtk_css_font_features_value_new_empty (void)
 {
   GtkCssValue *result;
 
-  result = _gtk_css_value_new (GtkCssValue, &GTK_CSS_VALUE_FONT_FEATURES);
+  result = gtk_css_value_new (GtkCssValue, &GTK_CSS_VALUE_FONT_FEATURES);
   result->features = g_hash_table_new_full (g_str_hash, g_str_equal, g_free, NULL);
   result->is_computed = TRUE;
 
@@ -183,7 +182,7 @@ gtk_css_font_features_value_new_default (void)
   if (default_font_features == NULL)
     default_font_features = gtk_css_font_features_value_new_empty ();
 
-  return _gtk_css_value_ref (default_font_features);
+  return gtk_css_value_ref (default_font_features);
 }
 
 static gboolean
@@ -217,7 +216,7 @@ gtk_css_font_features_value_parse (GtkCssParser *parser)
     name = gtk_css_parser_consume_string (parser);
     if (name == NULL)
       {
-        _gtk_css_value_unref (result);
+        gtk_css_value_unref (result);
         return NULL;
       }
 
@@ -225,7 +224,7 @@ gtk_css_font_features_value_parse (GtkCssParser *parser)
       {
         gtk_css_parser_error_value (parser, "Not a valid OpenType tag.");
         g_free (name);
-        _gtk_css_value_unref (result);
+        gtk_css_value_unref (result);
         return NULL;
       }
 
@@ -238,7 +237,7 @@ gtk_css_font_features_value_parse (GtkCssParser *parser)
         if (!gtk_css_parser_consume_integer (parser, &num))
           {
             g_free (name);
-            _gtk_css_value_unref (result);
+            gtk_css_value_unref (result);
             return NULL;
           }
       }
