@@ -27,6 +27,7 @@
 #include "gtkcssnumbervalueprivate.h"
 #include "gtkcsspositionvalueprivate.h"
 #include "gtkcssprovider.h"
+#include "gtksnapshotprivate.h"
 
 G_DEFINE_TYPE (GtkCssImageConic, gtk_css_image_conic, GTK_TYPE_CSS_IMAGE)
 
@@ -37,14 +38,15 @@ gtk_css_image_conic_snapshot (GtkCssImage        *image,
                               double              height)
 {
   GtkCssImageConic *self = GTK_CSS_IMAGE_CONIC (image);
-  GskColorStop *stops;
+  GskGradientStop *stops;
   int i, last;
-  double offset;
+  double offset, hint;
 
-  stops = g_newa (GskColorStop, self->n_stops);
+  stops = g_newa (GskGradientStop, self->n_stops);
 
   last = -1;
   offset = 0;
+  hint = 0;
   for (i = 0; i < self->n_stops; i++)
     {
       const GtkCssImageConicColorStop *stop = &self->color_stops[i];
@@ -52,6 +54,12 @@ gtk_css_image_conic_snapshot (GtkCssImage        *image,
 
       if (stop->offset == NULL)
         {
+          if (stop->transition_hint)
+            {
+              hint = MAX (hint, gtk_css_number_value_get (stop->transition_hint, 360) / 360);
+              hint = CLAMP (hint, 0.0, 1.0);
+            }
+
           if (i == 0)
             pos = 0.0;
           else if (i + 1 == self->n_stops)
@@ -61,10 +69,11 @@ gtk_css_image_conic_snapshot (GtkCssImage        *image,
         }
       else
         {
-          pos = _gtk_css_number_value_get (stop->offset, 360) / 360;
+          pos = gtk_css_number_value_get (stop->offset, 360) / 360;
           pos = CLAMP (pos, 0.0, 1.0);
         }
 
+      pos = MAX (pos, hint);
       pos = MAX (pos, offset);
       step = (pos - offset) / (i - last);
       for (last = last + 1; last <= i; last++)
@@ -74,56 +83,38 @@ gtk_css_image_conic_snapshot (GtkCssImage        *image,
           offset += step;
 
           stops[last].offset = offset;
-          stops[last].color = *gtk_css_color_value_get_rgba (stop->color);
+          gtk_css_color_to_color (gtk_css_color_value_get_color (stop->color), &stops[last].color);
+          if (last > 0 && stop->transition_hint)
+            {
+              hint = gtk_css_number_value_get (stop->transition_hint, 360) / 360;
+              hint = CLAMP (hint, 0.0, 1.0);
+              stops[last].transition_hint = (hint - stops[last - 1].offset) / (stops[last].offset - stops[last - 1].offset);
+            }
+          else
+            {
+              stops[last].transition_hint = 0.5;
+            }
         }
 
       offset = pos;
       last = i;
     }
 
-    gtk_snapshot_append_conic_gradient (
+  if (self->color_space != GTK_CSS_COLOR_SPACE_SRGB)
+    g_warning_once ("Gradient interpolation color spaces are not supported yet");
+
+  gtk_snapshot_add_conic_gradient (
           snapshot,
           &GRAPHENE_RECT_INIT (0, 0, width, height),
           &GRAPHENE_POINT_INIT (_gtk_css_position_value_get_x (self->center, width),
                                 _gtk_css_position_value_get_y (self->center, height)),
-          _gtk_css_number_value_get (self->rotation, 360),
-          stops,
-          self->n_stops);
-}
+          gtk_css_number_value_get (self->rotation, 360),
+          gtk_css_color_space_get_color_state (self->color_space),
+          gtk_css_hue_interpolation_to_hue_interpolation (self->hue_interp),
+          stops, self->n_stops);
 
-static gboolean
-parse_angles (GtkCssParser *parser,
-              gpointer      option_data,
-              gpointer      unused)
-{
-  GtkCssValue **angles = option_data;
-  
-  angles[0] = _gtk_css_number_value_parse (parser, GTK_CSS_PARSE_ANGLE | GTK_CSS_PARSE_PERCENT);
-  if (angles[0] == NULL)
-    return FALSE;
-
-  if (gtk_css_number_value_can_parse (parser))
-    {
-      angles[1] = _gtk_css_number_value_parse (parser, GTK_CSS_PARSE_ANGLE | GTK_CSS_PARSE_PERCENT);
-      if (angles[1] == NULL)
-        return FALSE;
-    }
-
-  return TRUE;
-}
-
-static gboolean
-parse_color (GtkCssParser *parser,
-             gpointer      option_data,
-             gpointer      unused)
-{
-  GtkCssValue **color = option_data;
-  
-  *color = _gtk_css_color_value_parse (parser);
-  if (*color == NULL)
-    return FALSE;
-
-  return TRUE;
+  for (i = 0; i < self->n_stops; i++)
+    gdk_color_finish (&stops[i].color);
 }
 
 static guint
@@ -131,39 +122,66 @@ gtk_css_image_conic_parse_color_stop (GtkCssImageConic *self,
                                       GtkCssParser      *parser,
                                       GArray            *stop_array)
 {
-  GtkCssValue *angles[2] = { NULL, NULL };
+  GtkCssValue *hint = NULL;
   GtkCssValue *color = NULL;
-  GtkCssParseOption options[] =
+  GtkCssValue *angles[2] = { NULL, NULL };
+
+  if (gtk_css_number_value_can_parse (parser))
     {
-      { (void *) gtk_css_number_value_can_parse, parse_angles, &angles },
-      { (void *) gtk_css_color_value_can_parse, parse_color, &color },
-    };
+      hint = gtk_css_number_value_parse (parser,
+                                         GTK_CSS_PARSE_PERCENT
+                                         | GTK_CSS_PARSE_ANGLE);
+      if (hint == NULL)
+        {
+          gtk_css_parser_error_syntax (parser, "Failed to parse transition hint");
+          goto fail;
+        }
 
-  if (!gtk_css_parser_consume_any (parser, options, G_N_ELEMENTS (options), NULL))
-    goto fail;
+      if (!gtk_css_parser_has_token (parser, GTK_CSS_TOKEN_COMMA))
+        goto fail;
 
+      gtk_css_parser_consume_token (parser);
+    }
+
+  color = gtk_css_color_value_parse (parser);
   if (color == NULL)
     {
-      gtk_css_parser_error_syntax (parser, "Expected shadow value to contain a length");
+      gtk_css_parser_error_syntax (parser, "Expected color stop to contain a color");
       goto fail;
     }
 
+  if (gtk_css_number_value_can_parse (parser))
+    {
+      angles[0] = gtk_css_number_value_parse (parser, GTK_CSS_PARSE_ANGLE | GTK_CSS_PARSE_PERCENT);
+      if (angles[0] == NULL)
+        goto fail;
+
+      if (gtk_css_number_value_can_parse (parser))
+        {
+          angles[1] = gtk_css_number_value_parse (parser, GTK_CSS_PARSE_ANGLE | GTK_CSS_PARSE_PERCENT);
+          if (angles[1] == NULL)
+            goto fail;
+        }
+    }
+
   g_array_append_vals (stop_array, (GtkCssImageConicColorStop[1]) {
-                         { angles[0], color }
+                         { angles[0], hint, color }
                        },
                        1);
   if (angles[1])
     g_array_append_vals (stop_array, (GtkCssImageConicColorStop[1]) {
-                           { angles[1], gtk_css_value_ref (color) }
+                           { angles[1], NULL, gtk_css_value_ref (color) }
                          },
                          1);
 
   return 1;
 
 fail:
+  g_clear_pointer (&hint, gtk_css_value_unref);
   g_clear_pointer (&angles[0], gtk_css_value_unref);
   g_clear_pointer (&angles[1], gtk_css_value_unref);
   g_clear_pointer (&color, gtk_css_value_unref);
+
   return 0;
 }
 
@@ -172,37 +190,57 @@ gtk_css_image_conic_parse_first_arg (GtkCssImageConic *self,
                                      GtkCssParser      *parser,
                                      GArray            *stop_array)
 {
-  gboolean nothing_parsed = TRUE;
+  gboolean has_rotation = FALSE;
+  gboolean has_center = FALSE;
+  gboolean has_colorspace = FALSE;
+  int retval = 1;
 
-  if (gtk_css_parser_try_ident (parser, "from"))
+  do
     {
-      self->rotation = _gtk_css_number_value_parse (parser, GTK_CSS_PARSE_ANGLE);
-      if (self->rotation == NULL)
-        return 0;
-      nothing_parsed = FALSE;
-    }
-  else
-    {
-      self->rotation = _gtk_css_number_value_new (0, GTK_CSS_DEG);
+      if (!has_colorspace && gtk_css_color_interpolation_method_can_parse (parser))
+        {
+          if (!gtk_css_color_interpolation_method_parse (parser, &self->color_space, &self->hue_interp))
+            return 0;
+          has_colorspace = TRUE;
+        }
+      else if (!has_rotation && gtk_css_parser_try_ident (parser, "from"))
+        {
+          self->rotation = gtk_css_number_value_parse (parser, GTK_CSS_PARSE_ANGLE);
+          if (self->rotation == NULL)
+            return 0;
+          has_rotation = TRUE;
+        }
+      else if (!has_center && gtk_css_parser_try_ident (parser, "at"))
+        {
+          self->center = _gtk_css_position_value_parse (parser);
+          if (self->center == NULL)
+            return 0;
+          has_center = TRUE;
+        }
+      else if (gtk_css_token_is (gtk_css_parser_get_token (parser), GTK_CSS_TOKEN_COMMA))
+        {
+          retval = 1;
+          break;
+        }
+      else
+        {
+          if (gtk_css_image_conic_parse_color_stop (self, parser, stop_array))
+            {
+              retval = 2;
+              break;
+            }
+        }
     }
+  while (!(has_colorspace && has_rotation && has_center));
 
-  if (gtk_css_parser_try_ident (parser, "at"))
-    {
-      self->center = _gtk_css_position_value_parse (parser);
-      if (self->center == NULL)
-        return 0;
-      nothing_parsed = FALSE;
-    }
-  else
-    {
-      self->center = _gtk_css_position_value_new (_gtk_css_number_value_new (50, GTK_CSS_PERCENT),
-                                                   _gtk_css_number_value_new (50, GTK_CSS_PERCENT));
-    }
+  if (!has_rotation)
+    self->rotation = gtk_css_number_value_new (0, GTK_CSS_DEG);
 
-  if (!nothing_parsed)
-    return 1;
+  if (!has_center)
+    self->center = _gtk_css_position_value_new (gtk_css_number_value_new (50, GTK_CSS_PERCENT),
+                                                gtk_css_number_value_new (50, GTK_CSS_PERCENT));
 
-  return 1 + gtk_css_image_conic_parse_color_stop (self, parser, stop_array);
+  return retval;
 }
 
 typedef struct
@@ -265,29 +303,40 @@ gtk_css_image_conic_print (GtkCssImage *image,
   gboolean written = FALSE;
   guint i;
 
-  g_string_append (string, "self-gradient(");
+  g_string_append (string, "conic-gradient(");
 
   if (self->center)
     {
-      GtkCssValue *compare = _gtk_css_position_value_new (_gtk_css_number_value_new (50, GTK_CSS_PERCENT),
-                                                          _gtk_css_number_value_new (50, GTK_CSS_PERCENT));
+      GtkCssValue *compare = _gtk_css_position_value_new (gtk_css_number_value_new (50, GTK_CSS_PERCENT),
+                                                          gtk_css_number_value_new (50, GTK_CSS_PERCENT));
 
-      if (!_gtk_css_value_equal (self->center, compare))
+      if (!gtk_css_value_equal (self->center, compare))
         {
-          g_string_append (string, "from ");
-          _gtk_css_value_print (self->center, string);
+          g_string_append (string, "at ");
+          gtk_css_value_print (self->center, string);
           written = TRUE;
         }
 
       gtk_css_value_unref (compare);
     }
 
-  if (self->rotation && _gtk_css_number_value_get (self->rotation, 360) != 0)
+  if (self->rotation && gtk_css_number_value_get (self->rotation, 360) != 0)
     {
       if (written)
         g_string_append_c (string, ' ');
-      g_string_append (string, "at ");
-      _gtk_css_value_print (self->rotation, string);
+      g_string_append (string, "from ");
+      gtk_css_value_print (self->rotation, string);
+      written = TRUE;
+    }
+
+  if (self->color_space != GTK_CSS_COLOR_SPACE_SRGB)
+    {
+      if (written)
+        g_string_append_c (string, ' ');
+      gtk_css_color_interpolation_method_print (self->color_space,
+                                                self->hue_interp,
+                                                string);
+      written = TRUE;
     }
 
   if (written)
@@ -300,12 +349,18 @@ gtk_css_image_conic_print (GtkCssImage *image,
       if (i > 0)
         g_string_append (string, ", ");
 
-      _gtk_css_value_print (stop->color, string);
+      if (stop->transition_hint)
+        {
+          gtk_css_value_print (stop->transition_hint, string);
+          g_string_append (string, ", ");
+        }
+
+      gtk_css_value_print (stop->color, string);
 
       if (stop->offset)
         {
           g_string_append (string, " ");
-          _gtk_css_value_print (stop->offset, string);
+          gtk_css_value_print (stop->offset, string);
         }
     }
 
@@ -313,11 +368,9 @@ gtk_css_image_conic_print (GtkCssImage *image,
 }
 
 static GtkCssImage *
-gtk_css_image_conic_compute (GtkCssImage      *image,
-                             guint             property_id,
-                             GtkStyleProvider *provider,
-                             GtkCssStyle      *style,
-                             GtkCssStyle      *parent_style)
+gtk_css_image_conic_compute (GtkCssImage          *image,
+                             guint                 property_id,
+                             GtkCssComputeContext *context)
 {
   GtkCssImageConic *self = GTK_CSS_IMAGE_CONIC (image);
   GtkCssImageConic *copy;
@@ -325,26 +378,29 @@ gtk_css_image_conic_compute (GtkCssImage      *image,
 
   copy = g_object_new (GTK_TYPE_CSS_IMAGE_CONIC, NULL);
 
-  copy->center = _gtk_css_value_compute (self->center, property_id, provider, style, parent_style);
-  copy->rotation = _gtk_css_value_compute (self->rotation, property_id, provider, style, parent_style);
+  copy->center = gtk_css_value_compute (self->center, property_id, context);
+  copy->rotation = gtk_css_value_compute (self->rotation, property_id, context);
+  copy->color_space = self->color_space;
+  copy->hue_interp = self->hue_interp;
 
   copy->n_stops = self->n_stops;
-  copy->color_stops = g_malloc (sizeof (GtkCssImageConicColorStop) * copy->n_stops);
+  copy->color_stops = g_new (GtkCssImageConicColorStop, self->n_stops);
   for (i = 0; i < self->n_stops; i++)
     {
       const GtkCssImageConicColorStop *stop = &self->color_stops[i];
       GtkCssImageConicColorStop *scopy = &copy->color_stops[i];
 
-      scopy->color = _gtk_css_value_compute (stop->color, property_id, provider, style, parent_style);
+      scopy->color = gtk_css_value_compute (stop->color, property_id, context);
 
       if (stop->offset)
-        {
-          scopy->offset = _gtk_css_value_compute (stop->offset, property_id, provider, style, parent_style);
-        }
+        scopy->offset = gtk_css_value_compute (stop->offset, property_id, context);
       else
-        {
-          scopy->offset = NULL;
-        }
+        scopy->offset = NULL;
+
+      if (stop->transition_hint)
+        scopy->transition_hint = gtk_css_value_compute (stop->transition_hint, property_id, context);
+      else
+        scopy->transition_hint = NULL;
     }
 
   return GTK_CSS_IMAGE (copy);
@@ -374,14 +430,17 @@ gtk_css_image_conic_transition (GtkCssImage *start_image,
 
   result = g_object_new (GTK_TYPE_CSS_IMAGE_CONIC, NULL);
 
-  result->center = _gtk_css_value_transition (start->center, end->center, property_id, progress);
+  result->center = gtk_css_value_transition (start->center, end->center, property_id, progress);
   if (result->center == NULL)
     goto fail;
 
-  result->rotation = _gtk_css_value_transition (start->rotation, end->rotation, property_id, progress);
+  result->rotation = gtk_css_value_transition (start->rotation, end->rotation, property_id, progress);
   if (result->rotation == NULL)
     goto fail;
 
+  result->color_space = start->color_space;
+  result->hue_interp = start->hue_interp;
+
   result->color_stops = g_malloc (sizeof (GtkCssImageConicColorStop) * start->n_stops);
   result->n_stops = 0;
   for (i = 0; i < start->n_stops; i++)
@@ -390,6 +449,23 @@ gtk_css_image_conic_transition (GtkCssImage *start_image,
       const GtkCssImageConicColorStop *end_stop = &end->color_stops[i];
       GtkCssImageConicColorStop *stop = &result->color_stops[i];
 
+      if ((start_stop->transition_hint != NULL) != (end_stop->transition_hint != NULL))
+        goto fail;
+
+      if (start_stop->transition_hint == NULL)
+        {
+          stop->transition_hint = NULL;
+        }
+      else
+        {
+          stop->transition_hint = gtk_css_value_transition (start_stop->transition_hint,
+                                                            end_stop->transition_hint,
+                                                            property_id,
+                                                            progress);
+          if (stop->transition_hint == NULL)
+            goto fail;
+        }
+
       if ((start_stop->offset != NULL) != (end_stop->offset != NULL))
         goto fail;
 
@@ -399,22 +475,22 @@ gtk_css_image_conic_transition (GtkCssImage *start_image,
         }
       else
         {
-          stop->offset = _gtk_css_value_transition (start_stop->offset,
-                                                    end_stop->offset,
-                                                    property_id,
-                                                    progress);
+          stop->offset = gtk_css_value_transition (start_stop->offset,
+                                                   end_stop->offset,
+                                                   property_id,
+                                                   progress);
           if (stop->offset == NULL)
             goto fail;
         }
 
-      stop->color = _gtk_css_value_transition (start_stop->color,
-                                               end_stop->color,
-                                               property_id,
-                                               progress);
+      stop->color = gtk_css_value_transition (start_stop->color,
+                                              end_stop->color,
+                                              property_id,
+                                              progress);
       if (stop->color == NULL)
         {
           if (stop->offset)
-            _gtk_css_value_unref (stop->offset);
+            gtk_css_value_unref (stop->offset);
           goto fail;
         }
 
@@ -436,8 +512,10 @@ gtk_css_image_conic_equal (GtkCssImage *image1,
   GtkCssImageConic *conic2 = (GtkCssImageConic *) image2;
   guint i;
 
-  if (!_gtk_css_value_equal (conic1->center, conic2->center) ||
-      !_gtk_css_value_equal (conic1->rotation, conic2->rotation))
+  if (!gtk_css_value_equal (conic1->center, conic2->center) ||
+      !gtk_css_value_equal (conic1->rotation, conic2->rotation) ||
+      conic1->color_space != conic2->color_space ||
+      conic1->hue_interp != conic2->hue_interp)
     return FALSE;
 
   for (i = 0; i < conic1->n_stops; i++)
@@ -445,8 +523,9 @@ gtk_css_image_conic_equal (GtkCssImage *image1,
       const GtkCssImageConicColorStop *stop1 = &conic1->color_stops[i];
       const GtkCssImageConicColorStop *stop2 = &conic2->color_stops[i];
 
-      if (!_gtk_css_value_equal0 (stop1->offset, stop2->offset) ||
-          !_gtk_css_value_equal (stop1->color, stop2->color))
+      if (!gtk_css_value_equal0 (stop1->offset, stop2->offset) ||
+          !gtk_css_value_equal0 (stop1->transition_hint, stop2->transition_hint) ||
+          !gtk_css_value_equal (stop1->color, stop2->color))
         return FALSE;
     }
 
@@ -463,9 +542,11 @@ gtk_css_image_conic_dispose (GObject *object)
     {
       GtkCssImageConicColorStop *stop = &self->color_stops[i];
 
-      _gtk_css_value_unref (stop->color);
+      gtk_css_value_unref (stop->color);
       if (stop->offset)
-        _gtk_css_value_unref (stop->offset);
+        gtk_css_value_unref (stop->offset);
+      if (stop->transition_hint)
+        gtk_css_value_unref (stop->transition_hint);
     }
   g_free (self->color_stops);
 
@@ -485,24 +566,84 @@ gtk_css_image_conic_is_computed (GtkCssImage *image)
   computed = !self->center || gtk_css_value_is_computed (self->center);
   computed &= !self->rotation || gtk_css_value_is_computed (self->rotation);
 
-  for (i = 0; i < self->n_stops; i ++)
+  if (computed)
+    for (i = 0; i < self->n_stops; i ++)
+      {
+        const GtkCssImageConicColorStop *stop = &self->color_stops[i];
+
+        if (stop->transition_hint && !gtk_css_value_is_computed (stop->transition_hint))
+          {
+            computed = FALSE;
+            break;
+          }
+
+        if (stop->offset && !gtk_css_value_is_computed (stop->offset))
+          {
+            computed = FALSE;
+            break;
+          }
+
+        if (!gtk_css_value_is_computed (stop->color))
+          {
+            computed = FALSE;
+            break;
+          }
+      }
+
+  return computed;
+}
+
+static gboolean
+gtk_css_image_conic_contains_current_color (GtkCssImage *image)
+{
+  GtkCssImageConic *self = GTK_CSS_IMAGE_CONIC (image);
+
+  for (guint i = 0; i < self->n_stops; i ++)
     {
       const GtkCssImageConicColorStop *stop = &self->color_stops[i];
 
-      if (stop->offset && !gtk_css_value_is_computed (stop->offset))
-        {
-          computed = FALSE;
-          break;
-        }
-
-      if (!gtk_css_value_is_computed (stop->color))
-        {
-          computed = FALSE;
-          break;
-        }
+      if (gtk_css_value_contains_current_color (stop->color))
+        return TRUE;
     }
 
-  return computed;
+  return FALSE;
+}
+
+static GtkCssImage *
+gtk_css_image_conic_resolve (GtkCssImage          *image,
+                             GtkCssComputeContext *context,
+                             GtkCssValue          *current_color)
+{
+  GtkCssImageConic *self = GTK_CSS_IMAGE_CONIC (image);
+  GtkCssImageConic *resolved;
+
+  if (!gtk_css_image_conic_contains_current_color (image))
+    return g_object_ref (image);
+
+  resolved = g_object_new (GTK_TYPE_CSS_IMAGE_CONIC, NULL);
+
+  resolved->center = gtk_css_value_ref (self->center);
+  resolved->rotation = gtk_css_value_ref (self->rotation);
+
+  resolved->n_stops = self->n_stops;
+  resolved->color_stops = g_new (GtkCssImageConicColorStop, self->n_stops);
+
+  for (guint i = 0; i < self->n_stops; i++)
+    {
+      if (self->color_stops[i].transition_hint)
+        resolved->color_stops[i].transition_hint = gtk_css_value_ref (self->color_stops[i].transition_hint);
+      else
+        resolved->color_stops[i].transition_hint = NULL;
+
+      if (self->color_stops[i].offset)
+        resolved->color_stops[i].offset = gtk_css_value_ref (self->color_stops[i].offset);
+      else
+        resolved->color_stops[i].offset = NULL;
+
+      resolved->color_stops[i].color = gtk_css_value_resolve (self->color_stops[i].color, context, current_color);
+    }
+
+  return GTK_CSS_IMAGE (resolved);
 }
 
 static void
@@ -518,6 +659,8 @@ gtk_css_image_conic_class_init (GtkCssImageConicClass *klass)
   image_class->equal = gtk_css_image_conic_equal;
   image_class->transition = gtk_css_image_conic_transition;
   image_class->is_computed = gtk_css_image_conic_is_computed;
+  image_class->contains_current_color = gtk_css_image_conic_contains_current_color;
+  image_class->resolve = gtk_css_image_conic_resolve;
 
   object_class->dispose = gtk_css_image_conic_dispose;
 }
