@@ -99,7 +99,7 @@ typedef enum
  * @GTK_ARROW_RIGHT: Represents a right pointing arrow.
  * @GTK_ARROW_NONE: No arrow.
  *
- * Used to indicate the direction in which an arrow should point.
+ * Indicates the direction in which an arrow should point.
  */
 typedef enum
 {
@@ -773,6 +773,8 @@ typedef enum
  * @GTK_TREE_VIEW_GRID_LINES_BOTH: Horizontal and vertical grid lines.
  *
  * Used to indicate which grid lines to draw in a tree view.
+ *
+ * Deprecated: 4.20: There is no replacement
  */
 typedef enum
 {
@@ -1043,9 +1045,10 @@ typedef enum
  * GtkPropagationLimit:
  * @GTK_LIMIT_NONE: Events are handled regardless of what their
  *   target is.
- * @GTK_LIMIT_SAME_NATIVE: Events are only handled if their target
- *   is in the same [iface@Native] as the event controllers widget. Note
- *   that some event types have two targets (origin and destination).
+ * @GTK_LIMIT_SAME_NATIVE: Events are only handled if their target is in
+ *   the same [iface@Native] (or widget with [property@Gtk.Widget:limit-events]
+ *   set) as the event controllers widget.
+ *   Note that some event types have two targets (origin and destination).
  *
  * Describes limits of a [class@EventController] for handling events
  * targeting other widgets.
@@ -1721,57 +1724,57 @@ typedef enum {
 
 /**
  * GTK_ACCESSIBLE_RELATION_LABEL_FOR:
- * 
+ *
  * Identifies the element (or elements) that are labeled by the
  * current element. Value type: reference
- * 
+ *
  * This relation is managed by GTK and should not be set from application code.
- * 
+ *
  * Since: 4.18
  */
 /**
  * GTK_ACCESSIBLE_RELATION_DESCRIPTION_FOR:
- * 
+ *
  * Identifies the element (or elements) that are described by
  * the current element. Value type: reference
- * 
+ *
  * This relation is managed by GTK and should not be set from application code.
- * 
+ *
  * Since: 4.18
  */
 /**
  * GTK_ACCESSIBLE_RELATION_CONTROLLED_BY:
- * 
+ *
  * Identifies the element (or elements) that the current
  * element is controlled by. Value type: reference
- * 
+ *
  * This relation is managed by GTK and should not be set from application code.
- * 
+ *
  * Since: 4.18
  */
 /**
  * GTK_ACCESSIBLE_RELATION_DETAILS_FOR:
- * 
+ *
  * Identifies the element (or elements) for which the current
  * element provides additional information. Value type: reference
- * 
+ *
  * This relation is managed by GTK and should not be set from application code.
- * 
+ *
  * Since: 4.18
  */
 /**
  * GTK_ACCESSIBLE_RELATION_ERROR_MESSAGE_FOR:
- * 
+ *
  * Identifies the element (or elements) for which the current
  * element provides an error message. Value type: reference
- * 
+ *
  * This relation is managed by GTK and should not be set from application code.
- * 
+ *
  * Since: 4.18
  */
 /**
  * GTK_ACCESSIBLE_RELATION_FLOW_FROM:
- * 
+ *
  * Identifies the previous element (or elements) in an alternate
  * reading order of content which, at the user's discretion, allows
  * assistive technology to override the general default of reading in
@@ -1975,4 +1978,55 @@ typedef enum {
   GTK_TEXT_BUFFER_NOTIFY_AFTER_DELETE  = 1 << 3,
 } GtkTextBufferNotifyFlags;
 
+/**
+ * GtkInterfaceColorScheme:
+ * @GTK_INTERFACE_COLOR_SCHEME_UNSUPPORTED: The system doesn't support color schemes
+ * @GTK_INTERFACE_COLOR_SCHEME_DEFAULT: The default color scheme is used
+ * @GTK_INTERFACE_COLOR_SCHEME_DARK: A dark color scheme is used
+ * @GTK_INTERFACE_COLOR_SCHEME_LIGHT: A light color scheme is used
+ *
+ * Values for the [property@Gtk.Settings:gtk-interface-color-scheme]
+ * and [property@Gtk.CssProvider:prefers-color-scheme] properties
+ * that indicates what color scheme is used.
+ *
+ * This information can be used inside CSS via media queries.
+ *
+ * More values may be added to this enumeration. Unknown values
+ * should be treated the same as `GTK_INTERFACE_COLOR_SCHEME_DEFAULT`.
+ *
+ * Since: 4.20
+ */
+typedef enum {
+  GTK_INTERFACE_COLOR_SCHEME_UNSUPPORTED,
+  GTK_INTERFACE_COLOR_SCHEME_DEFAULT,
+  GTK_INTERFACE_COLOR_SCHEME_DARK,
+  GTK_INTERFACE_COLOR_SCHEME_LIGHT,
+} GtkInterfaceColorScheme;
+
+/**
+ * GtkInterfaceContrast
+ * @GTK_INTERFACE_CONTRAST_UNSUPPORTED: The system doesn't support contrast levels
+ * @GTK_INTERFACE_CONTRAST_NO_PREFERENCE: No particular preference for contrast
+ * @GTK_INTERFACE_CONTRAST_MORE: More contrast is preferred
+ * @GTK_INTERFACE_CONTRAST_LESS: Less contrast is preferred
+ *
+ * Values for the [property@Gtk.Settings:gtk-interface-contrast]
+ * and [property@Gtk.CssProvider:prefers-contrast] properties
+ * that indicates the preferred level of contrast.
+ *
+ * This information can be used inside CSS via media queries.
+ *
+ * More values may be added to this enumeration. Unknown values
+ * should be treated the same as `GTK_INTERFACE_CONTRAST_NO_PREFERENCE`.
+ *
+ * Since: 4.20
+ */
+typedef enum {
+  GTK_INTERFACE_CONTRAST_UNSUPPORTED,
+  GTK_INTERFACE_CONTRAST_NO_PREFERENCE,
+  GTK_INTERFACE_CONTRAST_MORE,
+  GTK_INTERFACE_CONTRAST_LESS,
+} GtkInterfaceContrast;
+
+
 G_END_DECLS
