@@ -96,6 +96,7 @@ gtk_file_dialog_finalize (GObject *object)
   g_clear_object (&self->filters);
   g_clear_object (&self->default_filter);
   g_clear_object (&self->initial_folder);
+  g_clear_object (&self->initial_file);
   g_free (self->initial_name);
 
   G_OBJECT_CLASS (gtk_file_dialog_parent_class)->finalize (object);
@@ -1011,7 +1012,7 @@ finish_multiple_files_op (GtkFileDialog   *self,
  *
  * The file chooser dialog will be set up to select a single file.
  *
- * The @callback will be called when the dialog is dismissed.
+ * The @callback will be called when the dialog is closed.
  *
  * Since: 4.10
  */
@@ -1050,7 +1051,10 @@ gtk_file_dialog_open (GtkFileDialog       *self,
  *
  * Finishes the [method@Gtk.FileDialog.open] call.
  *
- * Returns: (nullable) (transfer full): the file that was selected
+ * Note that this function returns a [error@Gtk.DialogError.DISMISSED]
+ * error if the user cancels the dialog.
+ *
+ * Returns: (transfer full): the file that was selected
  *
  * Since: 4.10
  */
@@ -1083,7 +1087,7 @@ gtk_file_dialog_open_finish (GtkFileDialog   *self,
  * be opened in the parent directory of that folder, otherwise, it
  * will be in the directory [property@Gtk.FileDialog:initial-folder].
  *
- * The @callback will be called when the dialog is dismissed.
+ * The @callback will be called when the dialog is closed.
  *
  * Since: 4.10
  */
@@ -1122,7 +1126,10 @@ gtk_file_dialog_select_folder (GtkFileDialog       *self,
  *
  * Finishes the [method@Gtk.FileDialog.select_folder] call.
  *
- * Returns: (nullable) (transfer full): the folder that was selected
+ * Note that this function returns a [error@Gtk.DialogError.DISMISSED]
+ * error if the user cancels the dialog.
+ *
+ * Returns: (transfer full): the folder that was selected
  *
  * Since: 4.10
  */
@@ -1151,7 +1158,7 @@ gtk_file_dialog_select_folder_finish (GtkFileDialog  *self,
  *
  * The file chooser dialog will be save mode.
  *
- * The @callback will be called when the dialog is dismissed.
+ * The @callback will be called when the dialog is closed.
  *
  * Since: 4.10
  */
@@ -1190,7 +1197,10 @@ gtk_file_dialog_save (GtkFileDialog       *self,
  *
  * Finishes the [method@Gtk.FileDialog.save] call.
  *
- * Returns: (nullable) (transfer full): the file that was selected
+ * Note that this function returns a [error@Gtk.DialogError.DISMISSED]
+ * error if the user cancels the dialog.
+ *
+ * Returns: (transfer full): the file that was selected
  *
  * Since: 4.10
  */
@@ -1222,7 +1232,7 @@ gtk_file_dialog_save_finish (GtkFileDialog   *self,
  * The file chooser dialog will initially be opened in the directory
  * [property@Gtk.FileDialog:initial-folder].
  *
- * The @callback will be called when the dialog is dismissed.
+ * The @callback will be called when the dialog is closed.
  *
  * Since: 4.10
  */
@@ -1261,7 +1271,10 @@ gtk_file_dialog_open_multiple (GtkFileDialog       *self,
  *
  * Finishes the [method@Gtk.FileDialog.open] call.
  *
- * Returns: (nullable) (transfer full): the files that were selected,
+ * Note that this function returns a [error@Gtk.DialogError.DISMISSED]
+ * error if the user cancels the dialog.
+ *
+ * Returns: (transfer full): the files that were selected,
  *   as a list model of [iface@Gio.File]
  *
  * Since: 4.10
@@ -1295,7 +1308,7 @@ gtk_file_dialog_open_multiple_finish (GtkFileDialog   *self,
  * The file chooser dialog will initially be opened in the
  * directory [property@Gtk.FileDialog:initial-folder].
  *
- * The @callback will be called when the dialog is dismissed.
+ * The @callback will be called when the dialog is closed.
  *
  * Since: 4.10
  */
@@ -1334,7 +1347,10 @@ gtk_file_dialog_select_multiple_folders (GtkFileDialog       *self,
  *
  * Finishes the [method@Gtk.FileDialog.select_multiple_folders] call.
  *
- * Returns: (nullable) (transfer full): the folders that were selected,
+ * Note that this function returns a [error@Gtk.DialogError.DISMISSED]
+ * error if the user cancels the dialog.
+ *
+ * Returns: (transfer full): the folders that were selected,
  *   as a list model of [iface@Gio.File]
  *
  * Since: 4.10
@@ -1369,7 +1385,7 @@ gtk_file_dialog_select_multiple_folders_finish (GtkFileDialog   *self,
  * In contrast to [method@Gtk.FileDialog.open], this function
  * lets the user select the text encoding for the file, if possible.
  *
- * The @callback will be called when the dialog is dismissed.
+ * The @callback will be called when the dialog is closed.
  *
  * Since: 4.18
  */
@@ -1435,7 +1451,10 @@ G_GNUC_END_IGNORE_DEPRECATIONS
  * is suitable for passing to iconv_open(). Otherwise, it will
  * be `NULL`.
  *
- * Returns: (nullable) (transfer full): the file that was selected
+ * Note that this function returns a [error@Gtk.DialogError.DISMISSED]
+ * error if the user cancels the dialog.
+ *
+ * Returns: (transfer full): the file that was selected
  *
  * Since: 4.18
  */
@@ -1486,7 +1505,7 @@ gtk_file_dialog_open_text_file_finish (GtkFileDialog  *self,
  * In contrast to [method@Gtk.FileDialog.open], this function
  * lets the user select the text encoding for the files, if possible.
  *
- * The @callback will be called when the dialog is dismissed.
+ * The @callback will be called when the dialog is closed.
  *
  * Since: 4.18
  */
@@ -1546,7 +1565,10 @@ G_GNUC_END_IGNORE_DEPRECATIONS
  *
  * Finishes the [method@Gtk.FileDialog.open] call.
  *
- * Returns: (nullable) (transfer full): the files that were selected,
+ * Note that this function returns a [error@Gtk.DialogError.DISMISSED]
+ * error if the user cancels the dialog.
+ *
+ * Returns: (transfer full): the files that were selected,
  *   as a list model of [iface@Gio.File]
  *
  * Since: 4.18
@@ -1595,7 +1617,7 @@ gtk_file_dialog_open_multiple_text_files_finish (GtkFileDialog   *self,
  * lets the user select the text encoding and line endings for
  * the text file, if possible.
  *
- * The @callback will be called when the dialog is dismissed.
+ * The @callback will be called when the dialog is closed.
  *
  * Since: 4.18
  */
@@ -1679,8 +1701,10 @@ G_GNUC_END_IGNORE_DEPRECATIONS
  * The @line_ending will be set to one of "\n", "\r\n", "\r" or "",
  * where the latter means to preserve existing line endings.
  *
- * Returns: (nullable) (transfer full): the file that was selected.
- *   Otherwise, `NULL` is returned and @error is set
+ * Note that this function returns a [error@Gtk.DialogError.DISMISSED]
+ * error if the user cancels the dialog.
+ *
+ * Returns: (transfer full): the file that was selected.
  *
  * Since: 4.18
  */
