@@ -40,10 +40,12 @@ static void     update_button_sensitivity
 /**
  * GtkFontDialogButton:
  *
- * The `GtkFontDialogButton` is wrapped around a [class@Gtk.FontDialog]
- * and allows to open a font chooser dialog to change the font.
+ * Opens a font chooser dialog to select a font.
  *
- * ![An example GtkFontDialogButton](font-button.png)
+ * <picture>
+ *   <source srcset="font-button-dark.png" media="(prefers-color-scheme: dark)">
+ *   <img alt="An example GtkFontDialogButton" src="font-button.png">
+ * </picture>
  *
  * It is suitable widget for selecting a font in a preference dialog.
  *
@@ -294,7 +296,7 @@ gtk_font_dialog_button_class_init (GtkFontDialogButtonClass *class)
   widget_class->unroot = gtk_font_dialog_button_unroot;
 
   /**
-   * GtkFontDialogButton:dialog: (attributes org.gtk.Property.get=gtk_font_dialog_button_get_dialog org.gtk.Property.set=gtk_font_dialog_button_set_dialog)
+   * GtkFontDialogButton:dialog:
    *
    * The `GtkFontDialog` that contains parameters for
    * the font chooser dialog.
@@ -307,7 +309,7 @@ gtk_font_dialog_button_class_init (GtkFontDialogButtonClass *class)
                            G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkFontDialogButton:level: (attributes org.gtk.Property.get=gtk_font_dialog_button_get_level org.gtk.Property.set=gtk_font_dialog_button_set_level)
+   * GtkFontDialogButton:level:
    *
    * The level of detail for the font chooser dialog.
    */
@@ -318,7 +320,7 @@ gtk_font_dialog_button_class_init (GtkFontDialogButtonClass *class)
                          GTK_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkFontDialogButton:font-desc: (attributes org.gtk.Property.get=gtk_font_dialog_button_get_font_desc org.gtk.Property.set=gtk_font_dialog_button_set_font_desc)
+   * GtkFontDialogButton:font-desc:
    *
    * The selected font.
    *
@@ -337,7 +339,7 @@ gtk_font_dialog_button_class_init (GtkFontDialogButtonClass *class)
                           G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkFontDialogButton:font-features: (attributes org.gtk.Property.get=gtk_font_dialog_button_get_font_features org.gtk.Property.set=gtk_font_dialog_button_set_font_features)
+   * GtkFontDialogButton:font-features:
    *
    * The selected font features.
    *
@@ -355,7 +357,7 @@ gtk_font_dialog_button_class_init (GtkFontDialogButtonClass *class)
                            G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkFontDialogButton:language: (attributes org.gtk.Property.get=gtk_font_dialog_button_get_language org.gtk.Property.set=gtk_font_dialog_button_set_language)
+   * GtkFontDialogButton:language:
    *
    * The selected language for font features.
    *
@@ -373,7 +375,7 @@ gtk_font_dialog_button_class_init (GtkFontDialogButtonClass *class)
                           G_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkFontDialogButton:use-font: (attributes org.gtk.Property.get=gtk_font_dialog_button_get_use_font org.gtk.Property.set=gtk_font_dialog_button_set_use_font)
+   * GtkFontDialogButton:use-font:
    *
    * Whether the buttons label will be drawn in the selected font.
    */
@@ -383,7 +385,7 @@ gtk_font_dialog_button_class_init (GtkFontDialogButtonClass *class)
                             GTK_PARAM_READWRITE|G_PARAM_STATIC_STRINGS|G_PARAM_EXPLICIT_NOTIFY);
 
   /**
-   * GtkFontDialogButton:use-size: (attributes org.gtk.Property.get=gtk_font_dialog_button_get_use_size org.gtk.Property.set=gtk_font_dialog_button_set_use_size)
+   * GtkFontDialogButton:use-size:
    *
    * Whether the buttons label will use the selected font size.
    */
@@ -621,6 +623,9 @@ update_font_data (GtkFontDialogButton *self)
         }
     }
 
+  if (self->font_family == NULL)
+    return;
+
   for (unsigned i = 0; i < g_list_model_get_n_items (G_LIST_MODEL (self->font_family)); i++)
     {
       PangoFontFace *face = g_list_model_get_item (G_LIST_MODEL (self->font_family), i);
@@ -1089,4 +1094,4 @@ gtk_font_dialog_button_get_use_size (GtkFontDialogButton *self)
 
 /* }}} */
 
-/* vim:set foldmethod=marker expandtab: */
+/* vim:set foldmethod=marker: */
