@@ -20,8 +20,6 @@
 #include <glib.h>
 #include <gdk-pixbuf/gdk-pixdata.h>
 
-#if defined(G_ENABLE_DEBUG) || defined(BUILD_TOOLS)
-
 #define VERBOSE(x)
 
 #define check(name,condition) \
@@ -401,5 +399,3 @@ gtk_icon_cache_validate (CacheInfo *info)
 
   return TRUE;
 }
-
-#endif
