@@ -27,7 +27,7 @@
 /**
  * GtkIMContext:
  *
- * `GtkIMContext` defines the interface for GTK input methods.
+ * The interface for GTK input methods.
  *
  * `GtkIMContext` is used by GTK text input widgets like `GtkText`
  * to map from key events to Unicode character strings.
@@ -933,7 +933,7 @@ gtk_im_context_get_surrounding_with_selection (GtkIMContext  *context,
  * have deleted all the characters that were requested to be deleted.
  *
  * This function is used by an input method that wants to make
- * subsitutions in the existing text in response to new input.
+ * substitutions in the existing text in response to new input.
  * It is not useful for applications.
  *
  * Returns: %TRUE if the signal was handled.
@@ -1005,3 +1005,31 @@ gtk_im_context_set_property (GObject      *obj,
       break;
     }
 }
+
+/**
+ * gtk_im_context_activate_osk:
+ * @context: a `GtkIMContext`
+ * @event: (nullable): a [class@Gdk.Event]
+ *
+ * Requests the platform to show an on-screen keyboard for user input.
+ *
+ * This method will return %TRUE if this request was actually performed
+ * to the platform, other environmental factors may result in an on-screen
+ * keyboard effectively not showing up.
+ *
+ * Returns: %TRUE if an on-screen keyboard could be requested to the platform.
+ *
+ * Since: 4.14
+ **/
+gboolean
+gtk_im_context_activate_osk (GtkIMContext *context,
+			     GdkEvent     *event)
+{
+  g_return_val_if_fail (GTK_IS_IM_CONTEXT (context), FALSE);
+  g_return_val_if_fail (!event || GDK_IS_EVENT (event), FALSE);
+
+  if (!GTK_IM_CONTEXT_GET_CLASS (context)->activate_osk_with_event)
+    return FALSE;
+
+  return GTK_IM_CONTEXT_GET_CLASS (context)->activate_osk_with_event (context, event);
+}
