@@ -191,10 +191,12 @@ gtk_list_factory_widget_update (GtkListItemBase *base,
 
   if (priv->object)
     {
+      gpointer old_item = gtk_list_item_base_get_item (GTK_LIST_ITEM_BASE (self));
+
       gtk_list_item_factory_update (priv->factory,
                                     priv->object,
-                                    gtk_list_item_base_get_item (GTK_LIST_ITEM_BASE (self)) != NULL,
-                                    item != NULL,
+                                    item != old_item && old_item != NULL,
+                                    item != old_item && item != NULL,
                                     gtk_list_factory_widget_update_func,
                                     &update);
     }
@@ -465,6 +467,9 @@ gtk_list_factory_widget_click_gesture_released (GtkGestureClick      *gesture,
       state = gdk_event_get_modifier_state (event);
       extend = (state & GDK_SHIFT_MASK) != 0;
       modify = (state & GDK_CONTROL_MASK) != 0;
+#ifdef __APPLE__
+      modify = modify | ((state & GDK_META_MASK) != 0);
+#endif
 
       gtk_widget_activate_action (GTK_WIDGET (self),
                                   "list.select-item",
