@@ -28,8 +28,7 @@
 /**
  * GtkSingleSelection:
  *
- * `GtkSingleSelection` is a `GtkSelectionModel` that allows selecting a single
- * item.
+ * A selection model that allows selecting a single item.
  *
  * Note that the selection is *persistent* -- if the selected item is removed
  * and re-added in the same [signal@Gio.ListModel::items-changed] emission, it
@@ -164,7 +163,7 @@ gtk_single_selection_unselect_item (GtkSelectionModel *model,
 {
   GtkSingleSelection *self = GTK_SINGLE_SELECTION (model);
 
-  if (!self->can_unselect)
+  if (!self->can_unselect || self->autoselect)
     return FALSE;
 
   if (self->selected == position)
@@ -173,12 +172,21 @@ gtk_single_selection_unselect_item (GtkSelectionModel *model,
   return TRUE;
 }
 
+static gboolean
+gtk_single_selection_unselect_all (GtkSelectionModel *model)
+{
+  GtkSingleSelection *self = GTK_SINGLE_SELECTION (model);
+
+  return gtk_single_selection_unselect_item (model, self->selected);
+}
+
 static void
 gtk_single_selection_selection_model_init (GtkSelectionModelInterface *iface)
 {
   iface->is_selected = gtk_single_selection_is_selected; 
   iface->get_selection_in_range = gtk_single_selection_get_selection_in_range; 
   iface->select_item = gtk_single_selection_select_item; 
+  iface->unselect_all = gtk_single_selection_unselect_all;
   iface->unselect_item = gtk_single_selection_unselect_item; 
 }
 
@@ -426,7 +434,7 @@ gtk_single_selection_class_init (GtkSingleSelectionClass *klass)
   gobject_class->dispose = gtk_single_selection_dispose;
 
   /**
-   * GtkSingleSelection:autoselect: (attributes org.gtk.Property.get=gtk_single_selection_get_autoselect org.gtk.Property.set=gtk_single_selection_set_autoselect)
+   * GtkSingleSelection:autoselect:
    *
    * If the selection will always select an item.
    */
@@ -436,7 +444,7 @@ gtk_single_selection_class_init (GtkSingleSelectionClass *klass)
                           G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkSingleSelection:can-unselect: (attributes org.gtk.Property.get=gtk_single_selection_get_can_unselect org.gtk.Property.set=gtk_single_selection_set_can_unselect)
+   * GtkSingleSelection:can-unselect:
    *
    * If unselecting the selected item is allowed.
    */
@@ -458,7 +466,7 @@ gtk_single_selection_class_init (GtkSingleSelectionClass *klass)
                         G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkSingleSelection:model: (attributes org.gtk.Property.get=gtk_single_selection_get_model org.gtk.Property.set=gtk_single_selection_set_model)
+   * GtkSingleSelection:model:
    *
    * The model being managed.
    */
@@ -480,7 +488,7 @@ gtk_single_selection_class_init (GtkSingleSelectionClass *klass)
                        G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkSingleSelection:selected: (attributes org.gtk.Property.get=gtk_single_selection_get_selected org.gtk.Property.set=gtk_single_selection_set_selected)
+   * GtkSingleSelection:selected:
    *
    * Position of the selected item.
    */
@@ -490,7 +498,7 @@ gtk_single_selection_class_init (GtkSingleSelectionClass *klass)
                        G_PARAM_READWRITE | G_PARAM_EXPLICIT_NOTIFY | G_PARAM_STATIC_STRINGS);
 
   /**
-   * GtkSingleSelection:selected-item: (attributes org.gtk.Property.get=gtk_single_selection_get_selected_item)
+   * GtkSingleSelection:selected-item:
    *
    * The selected item.
    */
@@ -535,7 +543,7 @@ gtk_single_selection_new (GListModel *model)
 }
 
 /**
- * gtk_single_selection_get_model: (attributes org.gtk.Method.get_property=model)
+ * gtk_single_selection_get_model:
  * @self: a `GtkSingleSelection`
  *
  * Gets the model that @self is wrapping.
@@ -551,7 +559,7 @@ gtk_single_selection_get_model (GtkSingleSelection *self)
 }
 
 /**
- * gtk_single_selection_set_model: (attributes org.gtk.Method.set_property=model)
+ * gtk_single_selection_set_model:
  * @self: a `GtkSingleSelection`
  * @model: (nullable): A `GListModel` to wrap
  *
@@ -613,7 +621,7 @@ gtk_single_selection_set_model (GtkSingleSelection *self,
 }
 
 /**
- * gtk_single_selection_get_selected: (attributes org.gtk.Method.get_property=selected)
+ * gtk_single_selection_get_selected:
  * @self: a `GtkSingleSelection`
  *
  * Gets the position of the selected item.
@@ -631,7 +639,7 @@ gtk_single_selection_get_selected (GtkSingleSelection *self)
 }
 
 /**
- * gtk_single_selection_set_selected: (attributes org.gtk.Method.set_property=selected)
+ * gtk_single_selection_set_selected:
  * @self: a `GtkSingleSelection`
  * @position: the item to select or %GTK_INVALID_LIST_POSITION
  *
@@ -642,7 +650,8 @@ gtk_single_selection_get_selected (GtkSingleSelection *self)
  * value of the [property@Gtk.SingleSelection:autoselect] property:
  * If it is set, no change will occur and the old item will stay
  * selected. If it is unset, the selection will be unset and no item
- * will be selected.
+ * will be selected. This also applies if [property@Gtk.SingleSelection:can-unselect]
+ * is set to %FALSE.
  */
 void
 gtk_single_selection_set_selected (GtkSingleSelection *self,
@@ -660,7 +669,12 @@ gtk_single_selection_set_selected (GtkSingleSelection *self,
     new_selected = g_list_model_get_item (self->model, position);
 
   if (new_selected == NULL)
-    position = GTK_INVALID_LIST_POSITION;
+    {
+      if (!self->can_unselect || self->autoselect)
+        return;
+
+      position = GTK_INVALID_LIST_POSITION;
+    }
 
   if (self->selected == position)
     return;
@@ -684,7 +698,7 @@ gtk_single_selection_set_selected (GtkSingleSelection *self,
 }
 
 /**
- * gtk_single_selection_get_selected_item: (attributes org.gtk.Method.get_property=selected-item)
+ * gtk_single_selection_get_selected_item:
  * @self: a `GtkSingleSelection`
  *
  * Gets the selected item.
@@ -702,7 +716,7 @@ gtk_single_selection_get_selected_item (GtkSingleSelection *self)
 }
 
 /**
- * gtk_single_selection_get_autoselect: (attributes org.gtk.Method.get_property=autoselect)
+ * gtk_single_selection_get_autoselect:
  * @self: a `GtkSingleSelection`
  *
  * Checks if autoselect has been enabled or disabled via
@@ -719,7 +733,7 @@ gtk_single_selection_get_autoselect (GtkSingleSelection *self)
 }
 
 /**
- * gtk_single_selection_set_autoselect: (attributes org.gtk.Method.set_property=autoselect)
+ * gtk_single_selection_set_autoselect:
  * @self: a `GtkSingleSelection`
  * @autoselect: %TRUE to always select an item
  *
@@ -751,7 +765,7 @@ gtk_single_selection_set_autoselect (GtkSingleSelection *self,
 }
 
 /**
- * gtk_single_selection_get_can_unselect: (attributes org.gtk.Method.get_property=can-unselect)
+ * gtk_single_selection_get_can_unselect:
  * @self: a `GtkSingleSelection`
  *
  * If %TRUE, gtk_selection_model_unselect_item() is supported and allows
@@ -768,7 +782,7 @@ gtk_single_selection_get_can_unselect (GtkSingleSelection *self)
 }
 
 /**
- * gtk_single_selection_set_can_unselect: (attributes org.gtk.Method.set_property=can-unselect)
+ * gtk_single_selection_set_can_unselect:
  * @self: a `GtkSingleSelection`
  * @can_unselect: %TRUE to allow unselecting
  *
@@ -777,7 +791,7 @@ gtk_single_selection_get_can_unselect (GtkSingleSelection *self)
  *
  * Note that setting [property@Gtk.SingleSelection:autoselect] will
  * cause unselecting to not work, so it practically makes no sense
- * to set both at the same time the same time.
+ * to set both at the same time.
  */
 void
 gtk_single_selection_set_can_unselect (GtkSingleSelection *self,
