@@ -201,6 +201,10 @@ struct _GtkInspectorA11y
 
   GtkWidget *box;
   GtkWidget *role;
+  GtkWidget *name;
+  GtkWidget *description;
+  GtkWidget *bounds;
+  GtkWidget *path_label;
   GtkWidget *path;
   GtkWidget *attributes;
 };
@@ -227,11 +231,43 @@ update_role (GtkInspectorA11y *sl)
   g_type_class_unref (eclass);
 }
 
+static void
+update_name (GtkInspectorA11y *sl)
+{
+  GtkATContext *context;
+  char *name;
+
+  context = gtk_accessible_get_at_context (GTK_ACCESSIBLE (sl->object));
+  if (context == NULL)
+    return;
+
+  name = gtk_at_context_get_name (context);
+  gtk_label_set_label (GTK_LABEL (sl->name), name);
+
+  g_object_unref (context);
+}
+
+static void
+update_description (GtkInspectorA11y *sl)
+{
+  GtkATContext *context;
+  char *description;
+
+  context = gtk_accessible_get_at_context (GTK_ACCESSIBLE (sl->object));
+  if (context == NULL)
+    return;
+
+  description = gtk_at_context_get_description (context);
+  gtk_label_set_label (GTK_LABEL (sl->description), description);
+
+  g_object_unref (context);
+}
+
 static void
 update_path (GtkInspectorA11y *sl)
 {
-  const char *path = "—";
 #ifdef G_OS_UNIX
+  const char *path = NULL;
   GtkATContext *context;
 
   context = gtk_accessible_get_at_context (GTK_ACCESSIBLE (sl->object));
@@ -242,13 +278,28 @@ update_path (GtkInspectorA11y *sl)
       else
         path = "not realized";
     }
-  else
-    path = "not on bus";
+
+  if (path != NULL)
+      gtk_label_set_label (GTK_LABEL (sl->path), path);
+
+  gtk_widget_set_visible (sl->path, path != NULL);
+  gtk_widget_set_visible (sl->path_label, path != NULL);
 
   g_clear_object (&context);
 #endif
+}
 
-  gtk_label_set_label (GTK_LABEL (sl->path), path);
+static void
+update_bounds (GtkInspectorA11y *sl)
+{
+  int x, y, w, h;
+
+  if (gtk_accessible_get_bounds (GTK_ACCESSIBLE (sl->object), &x, &y, &w, &h))
+    {
+      char *size_label = g_strdup_printf ("%d × %d +%d +%d", w, h, x, y);
+      gtk_label_set_label (GTK_LABEL (sl->bounds), size_label);
+      g_free (size_label);
+    }
 }
 
 extern GType gtk_string_pair_get_type (void);
@@ -361,6 +412,8 @@ setup_cell_cb (GtkSignalListItemFactory *factory,
 
   label = gtk_label_new (NULL);
   gtk_label_set_xalign (GTK_LABEL (label), 0.0);
+  gtk_label_set_ellipsize (GTK_LABEL (label), PANGO_ELLIPSIZE_END);
+  gtk_label_set_width_chars (GTK_LABEL (label), 50);
   gtk_widget_set_margin_start (label, 6);
   gtk_widget_set_margin_end (label, 6);
   gtk_list_item_set_child (list_item, label);
@@ -409,6 +462,8 @@ static void
 refresh_all (GtkInspectorA11y *sl)
 {
   update_role (sl);
+  update_name (sl);
+  update_description (sl);
   update_path (sl);
   update_attributes (sl);
 }
@@ -441,14 +496,14 @@ gtk_inspector_a11y_set_object (GtkInspectorA11y *sl,
       context = gtk_accessible_get_at_context (GTK_ACCESSIBLE (sl->object));
       if (context != NULL)
         {
+          gtk_at_context_realize (context);
           g_signal_connect_swapped (context, "state-change", G_CALLBACK (refresh_all), sl);
           g_object_unref (context);
         }
 
       gtk_stack_page_set_visible (page, TRUE);
-      update_role (sl);
-      update_path (sl);
-      update_attributes (sl);
+      refresh_all (sl);
+      update_bounds (sl);
     }
   else
     {
@@ -460,6 +515,11 @@ static void
 gtk_inspector_a11y_init (GtkInspectorA11y *sl)
 {
   gtk_widget_init_template (GTK_WIDGET (sl));
+
+#ifndef G_OS_UNIX
+  gtk_widget_set_visible (sl->path, FALSE);
+  gtk_widget_set_visible (sl->path_label, FALSE);
+#endif
 }
 
 static void
@@ -497,6 +557,10 @@ gtk_inspector_a11y_class_init (GtkInspectorA11yClass *klass)
   gtk_widget_class_set_template_from_resource (widget_class, "/org/gtk/libgtk/inspector/a11y.ui");
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorA11y, box);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorA11y, role);
+  gtk_widget_class_bind_template_child (widget_class, GtkInspectorA11y, name);
+  gtk_widget_class_bind_template_child (widget_class, GtkInspectorA11y, description);
+  gtk_widget_class_bind_template_child (widget_class, GtkInspectorA11y, bounds);
+  gtk_widget_class_bind_template_child (widget_class, GtkInspectorA11y, path_label);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorA11y, path);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorA11y, attributes);
 
