@@ -65,6 +65,7 @@ struct _GtkInspectorLogs
   GtkWidget *vulkan_gsk;
   GtkWidget *shaders;
   GtkWidget *cache;
+  GtkWidget *fallback;
   GtkWidget *verbose;
 
   GtkWidget *actions;
@@ -144,6 +145,7 @@ flag_toggled (GtkWidget        *button,
   update_flag (logs->vulkan_gsk, &flags, GSK_DEBUG_VULKAN);
   update_flag (logs->shaders, &flags, GSK_DEBUG_SHADERS);
   update_flag (logs->cache, &flags, GSK_DEBUG_CACHE);
+  update_flag (logs->fallback, &flags, GSK_DEBUG_FALLBACK);
   update_flag (logs->verbose, &flags, GSK_DEBUG_VERBOSE);
   gsk_set_debug_flags (flags);
 
@@ -206,6 +208,7 @@ gtk_inspector_logs_class_init (GtkInspectorLogsClass *klass)
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorLogs, vulkan_gsk);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorLogs, shaders);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorLogs, cache);
+  gtk_widget_class_bind_template_child (widget_class, GtkInspectorLogs, fallback);
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorLogs, verbose);
 
   gtk_widget_class_bind_template_child (widget_class, GtkInspectorLogs, actions);
