@@ -22,7 +22,7 @@ overlay_get_child_position (GtkOverlay *overlay,
   return TRUE;
 }
 
-int 
+int
 main (int argc, char *argv[])
 {
   GtkWidget *win, *overlay, *grid, *main_child, *child, *label, *sw;
@@ -32,13 +32,12 @@ main (int argc, char *argv[])
   gtk_init ();
 
   provider = gtk_css_provider_new ();
-  gtk_css_provider_load_from_data (provider,
-                                   "label { border: 3px solid black; border-radius: 5px; padding: 2px; }"
-                                   ".top { border-top-style: none; border-top-right-radius: 0px; border-top-left-radius: 0px; }"
-                                   ".bottom { border-bottom-style: none; border-bottom-right-radius: 0px; border-bottom-left-radius: 0px; }"
-                                   ".left { border-left-style: none; border-top-left-radius: 0px; border-bottom-left-radius: 0px; }"
-                                   ".right { border-right-style: none; border-top-right-radius: 0px; border-bottom-right-radius: 0px; }",
-                                   -1);
+  gtk_css_provider_load_from_string (provider,
+    "label { border: 3px solid black; border-radius: 5px; padding: 2px; }"
+    ".top { border-top-style: none; border-top-right-radius: 0px; border-top-left-radius: 0px; }"
+    ".bottom { border-bottom-style: none; border-bottom-right-radius: 0px; border-bottom-left-radius: 0px; }"
+    ".left { border-left-style: none; border-top-left-radius: 0px; border-bottom-left-radius: 0px; }"
+    ".right { border-right-style: none; border-top-right-radius: 0px; border-bottom-right-radius: 0px; }");
   gtk_style_context_add_provider_for_display (gdk_display_get_default (),
                                               GTK_STYLE_PROVIDER (provider),
                                               GTK_STYLE_PROVIDER_PRIORITY_APPLICATION);
